
#ifndef KWEATHERCORE_EXPORT_H
#define KWEATHERCORE_EXPORT_H

#ifdef KWEATHERCORE_STATIC_DEFINE
#  define KWEATHERCORE_EXPORT
#  define KWEATHERCORE_NO_EXPORT
#else
#  ifndef KWEATHERCORE_EXPORT
#    ifdef KWeatherCore_EXPORTS
        /* We are building this library */
#      define KWEATHERCORE_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KWEATHERCORE_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KWEATHERCORE_NO_EXPORT
#    define KWEATHERCORE_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KWEATHERCORE_DEPRECATED
#  define KWEATHERCORE_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KWEATHERCORE_DEPRECATED_EXPORT
#  define KWEATHERCORE_DEPRECATED_EXPORT KWEATHERCORE_EXPORT KWEATHERCORE_DEPRECATED
#endif

#ifndef KWEATHERCORE_DEPRECATED_NO_EXPORT
#  define KWEATHERCORE_DEPRECATED_NO_EXPORT KWEATHERCORE_NO_EXPORT KWEATHERCORE_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KWEATHERCORE_NO_DEPRECATED
#    define KWEATHERCORE_NO_DEPRECATED
#  endif
#endif

#endif /* KWEATHERCORE_EXPORT_H */
