// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KWEATHERCORE_VERSION_H
#define KWEATHERCORE_VERSION_H

#define KWEATHERCORE_VERSION_STRING "24.12.3"
#define KWEATHERCORE_VERSION_MAJOR 24
#define KWEATHERCORE_VERSION_MINOR 12
#define KWEATHERCORE_VERSION_PATCH 3
#define KWEATHERCORE_VERSION ((24<<16)|(12<<8)|(3))

#endif
